from django.conf import settings

PERMISSIONS_AUDITOR_PROCESSORS = [
    'permissions_auditor.processors.auth_mixins.PermissionRequiredMixinProcessor',
    'permissions_auditor.processors.auth_mixins.LoginRequiredMixinProcessor',
    'permissions_auditor.processors.auth_mixins.UserPassesTestMixinProcessor',
    'permissions_auditor.processors.auth_decorators.PermissionRequiredDecoratorProcessor',
    'permissions_auditor.processors.auth_decorators.LoginRequiredDecoratorProcessor',
    'permissions_auditor.processors.auth_decorators.StaffMemberRequiredDecoratorProcessor',
    'permissions_auditor.processors.auth_decorators.SuperUserRequiredDecoratorProcessor',
    'permissions_auditor.processors.auth_decorators.UserPassesTestDecoratorProcessor',
]
PERMISSIONS_AUDITOR_BLACKLIST = {
    'namespaces': [
        'admin',
    ],
    'view_names': [],
    'modules': [],
}
PERMISSIONS_AUDITOR_ADMIN = True
PERMISSIONS_AUDITOR_ROOT_URLCONF = settings.ROOT_URLCONF

PERMISSIONS_AUDITOR_CACHE_KEY = 'permissions_auditor_views'
PERMISSIONS_AUDITOR_CACHE_TIMEOUT = 900



"""Processors for django auth decorators."""

import inspect

from .base import BaseFuncViewProcessor


class PermissionRequiredDecoratorProcessor(BaseFuncViewProcessor):
    """
    Process ``@permission_required()`` decorator on function based views.
    """

    def can_process(self, view):
        if not super().can_process(view):
            return False

        # Unwrap the function and look for the has_perms() function.
        closures = inspect.getclosurevars(view).nonlocals
        if 'test_func' in closures:
            test_closures = inspect.getclosurevars(closures['test_func']).unbound
            if 'has_perms' in test_closures:
                return True

        return False

    def get_permission_required(self, view):
        permissions = []

        # Unwrap the function and search for the permission passed through
        # the @permission_required() decorator.
        closures = inspect.getclosurevars(view).nonlocals['test_func']
        test_func_closures = inspect.getclosurevars(closures).nonlocals
        if 'perm' in test_func_closures:
            perm = test_func_closures['perm']

            # Ensure perm is not a function
            if not inspect.isfunction(perm):
                if isinstance(perm, str):
                    permissions.append(perm)
                else:
                    permissions.extend(perm)

        return permissions

    def get_login_required(self, view):
        return True


class LoginRequiredDecoratorProcessor(BaseFuncViewProcessor):
    """
    Process ``@login_required`` decorator on function based views.
    """

    def can_process(self, view):
        if not super().can_process(view):
            return False

        # Unwrap the function and look for the is_authenticated property.
        closures = inspect.getclosurevars(view).nonlocals
        if 'test_func' in closures:
            test_closures = inspect.getclosurevars(closures['test_func']).unbound
            if 'is_authenticated' in test_closures:
                return True

        return False

    def get_login_required(self, view):
        return True


class StaffMemberRequiredDecoratorProcessor(BaseFuncViewProcessor):
    """
    Process Django admin's ``@staff_member_required`` decorator on function based views.
    """

    def can_process(self, view):
        if not super().can_process(view):
            return False

        # Unwrap the function and look for the is_staff property.
        closures = inspect.getclosurevars(view).nonlocals
        if 'test_func' in closures:
            test_closures = inspect.getclosurevars(closures['test_func']).unbound
            if 'is_staff' in test_closures:
                return True

        return False

    def get_login_required(self, view):
        return True

    def get_docstring(self, view):
        return 'Staff member required'


class ActiveUserRequiredDecoratorProcessor(BaseFuncViewProcessor):
    """
    Process ``@user_passes_test(lambda u: u.is_active)`` decorator on function based views.
    """

    def can_process(self, view):
        if not super().can_process(view):
            return False

        # Unwrap the function and look for the is_active property.
        closures = inspect.getclosurevars(view).nonlocals
        if 'test_func' in closures:
            test_closures = inspect.getclosurevars(closures['test_func']).unbound
            if 'is_active' in test_closures:
                return True

        return False

    def get_login_required(self, view):
        return True

    def get_docstring(self, view):
        return 'Active user required'


class AnonymousUserRequiredDecoratorProcessor(BaseFuncViewProcessor):
    """
    Process ``@user_passes_test(lambda u: u.is_anonymous)`` decorator on function based views.
    """

    def can_process(self, view):
        if not super().can_process(view):
            return False

        # Unwrap the function and look for the is_anonymous property.
        closures = inspect.getclosurevars(view).nonlocals
        if 'test_func' in closures:
            test_closures = inspect.getclosurevars(closures['test_func']).unbound
            if 'is_anonymous' in test_closures:
                return True

        return False

    def get_docstring(self, view):
        return 'Anonymous user required'


class SuperUserRequiredDecoratorProcessor(BaseFuncViewProcessor):
    """
    Process ``@user_passes_test(lambda u: u.is_superuser)`` decorator on function based views.
    """

    def can_process(self, view):
        if not super().can_process(view):
            return False

        # Unwrap the function and look for the is_superuser property.
        closures = inspect.getclosurevars(view).nonlocals
        if 'test_func' in closures:
            test_closures = inspect.getclosurevars(closures['test_func']).unbound
            if 'is_superuser' in test_closures:
                return True

        return False

    def get_login_required(self, view):
        return True

    def get_docstring(self, view):
        return 'Superuser required'


class UserPassesTestDecoratorProcessor(BaseFuncViewProcessor):
    """
    Process ``@user_passes_test()`` decorator on function based views.

    .. note::
        the ``@user_passes_test`` decorator does not automatically check
        that the User is not anonymous. This means they don't necessarily need
        to be authenticated for the check to pass, so this processor returns
        ``None`` (unknown) for the login_required attribute.
    """

    def can_process(self, view):
        if not super().can_process(view):
            return False

        # The other decorators build from the user_passes_test decorator,
        # so we need to blacklist their functions so we don't override their results.
        blacklist = (
            'is_authenticated', 'has_perms', 'is_staff', 'is_active', 'is_anonymous',
            'is_superuser',
        )

        # Unwrap the function and look for any test functions inside the decorator.
        closures = inspect.getclosurevars(view).nonlocals
        if 'test_func' in closures:
            test_closures = inspect.getclosurevars(closures['test_func']).unbound
            return any([closure not in blacklist for closure in test_closures])

        return False

    def get_login_required(self, view):
        return None

    def get_docstring(self, view):
        return 'Custom user test'



from django.contrib import admin
from django.contrib.admin import helpers
from django.contrib.auth.models import Permission
from django.db import models
from django.template.response import TemplateResponse
from django.urls import path
from django.utils.translation import gettext_lazy as _

from permissions_auditor.core import get_views, _get_setting
from permissions_auditor.forms import AuditorAdminPermissionForm


class Index(models.Model):
    """Dummy model to display our pages in the admin."""
    class Meta:
        verbose_name = 'permission'
        verbose_name_plural = _('Site Views Index')
        app_label = 'permissions_auditor'


class ViewIndex(admin.ModelAdmin):
    form = AuditorAdminPermissionForm
    fieldsets = (
        (_('Permission Info'), {
            'fields': ('name', 'content_type', 'codename'),
        }),
        (_('Objects with this Permission'), {
            'fields': ('users', 'groups'),
        }),
    )

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name
        return [
            path('', self.admin_site.admin_view(self.index), name='%s_%s_changelist' % info),
            path('<str:permission>/',
                 self.admin_site.admin_view(self.permission_detail),
                 name='%s_%s_permissiondetail' % info),
        ]

    def get_object(self, request, permission, from_field=None):
        app_label, codename = permission.split('.')
        try:
            return Permission.objects.get(content_type__app_label=app_label, codename=codename)
        except (Permission.DoesNotExist, ValueError):
            return None

    def get_form(self, request, obj, change=False, **kwargs):
        defaults = {
            'users': obj.user_set.filter(is_active=True),
            'groups': obj.group_set.all()
        }
        return self.form(request.POST or defaults, instance=obj)

    def index(self, request):
        context = dict(self.admin_site.each_context(request))

        context.update({
            'views': get_views(),
            'group_by': request.GET.get('group_by', 'module')
        })
        return TemplateResponse(request, "permissions_auditor/admin/views_index.html", context)

    def permission_detail(self, request, permission, obj=None):
        obj = self.get_object(request, permission)
        if obj is None:
            return self._get_obj_does_not_exist_redirect(request, self.model._meta, permission)

        opts = self.model._meta

        adminForm = helpers.AdminForm(
            self.get_form(request, obj),
            list(self.get_fieldsets(request, obj)),
            {},
            model_admin=self
        )
        media = self.media + adminForm.media

        if (request.method == 'POST' and
                adminForm.form.is_valid() and
                self.has_auditor_change_permission(request)):
            obj.user_set.set(adminForm.form.cleaned_data['users'])
            obj.group_set.set(adminForm.form.cleaned_data['groups'])
            return self.response_change(request, obj)

        context = {
            **self.admin_site.each_context(request),
            'adminform': adminForm,
            'errors': helpers.AdminErrorList(adminForm.form, []),
            'media': media,

            'views': get_views(),
            'permission': '{}.{}'.format(obj.content_type.app_label, obj.codename),

            'opts': opts,
            'add': False,
            'change': True,
            'is_popup': False,
            'save_as': self.save_as,
            'has_editable_inline_admin_formsets': False,
            'has_view_permission': self.has_view_permission(request, obj),
            'has_add_permission': self.has_add_permission(request, obj),
            'has_change_permission': self.has_auditor_change_permission(request),
            'has_delete_permission': self.has_delete_permission(request, obj),
            'app_label': opts.app_label,
        }

        return TemplateResponse(
            request, "permissions_auditor/admin/permission_detail.html", context
        )

    def has_view_permission(self, request, obj=None):
        return request.user.is_staff

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_module_permission(self, request):
        return request.user.is_staff

    def has_auditor_change_permission(self, request):
        return request.user.has_perms(['auth.change_user', 'auth.change_group'])


if _get_setting('PERMISSIONS_AUDITOR_ADMIN'):
    admin.site.register(Index, ViewIndex)



"""Views used for testing."""
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import (
    login_required, permission_required, user_passes_test
)
from django.contrib.auth.mixins import (
    LoginRequiredMixin, PermissionRequiredMixin, UserPassesTestMixin
)
from django.views.generic import View


# Class Based Views


class BaseView(View):
    pass


class LoginRequiredView(LoginRequiredMixin, View):
    pass


class PermissionRequiredView(PermissionRequiredMixin, View):
    permission_required = 'tests.test_perm'


class PermissionRequiredMultiView(PermissionRequiredMixin, View):
    permission_required = ('tests.test_perm', 'tests.test_perm2')


class PermissionRequiredViewNoPerm(PermissionRequiredMixin, View):

    def has_permission(self):
        """The user's first name must be Bob"""
        return self.request.user.first_name == 'Bob'


class PermissionRequiredViewDocstring(PermissionRequiredMixin, View):
    permission_required = 'tests.test_perm'

    def has_permission(self):
        """Custom docstrings should be detected."""
        return super().has_permission()


class PermissionRequiredViewNoDocstring(PermissionRequiredMixin, View):
    permission_required = 'tests.test_perm'

    def has_permission(self):
        return super().has_permission()


class UserPassesTestView(UserPassesTestMixin, View):
    def test_func(self):
        return True


class UserPassesTestViewDocstring(UserPassesTestMixin, View):
    def test_func(self):
        """Custom docstrings should be detected."""
        return True


class UserPassesTestViewNoDocstring(UserPassesTestMixin, View):
    def test_func(self):
        return True


class UserPassesTestViewCustomFunc(UserPassesTestMixin, View):
    def get_test_func(self):
        return self.custom_test_func

    def custom_test_func(self):
        """Custom docstrings should be detected."""
        return True


# Function Based Views


def base_view(request):
    pass


@login_required
def login_required_view(request):
    pass


@permission_required('tests.test_perm')
def permission_required_view(request):
    pass


@permission_required(('tests.test_perm', 'tests.test_perm2'))
def permission_required_multi_view(request):
    pass


@staff_member_required
def staff_member_required_view(request):
    pass


@user_passes_test(lambda u: u.is_active)
def active_user_required_view(request):
    pass


@user_passes_test(lambda u: u.is_anonymous)
def anonymous_user_required_view(request):
    pass


@user_passes_test(lambda u: u.is_superuser)
def superuser_required_view(request):
    pass


@user_passes_test(lambda u: u.email is not None)
def user_passes_test_view(request):
    pass



from permissions_auditor.processors import auth_decorators
from permissions_auditor.tests.base import ProcessorTestCase
from permissions_auditor.tests.fixtures import views


class DecoratorProcessorTestCaseMixin:
    """Decorators should never be able to process class based views."""

    def test_cb_baseview(self):
        self.assertProcessorResults(views.BaseView, can_process=False)

    def test_cb_loginrequiredview(self):
        self.assertProcessorResults(views.LoginRequiredView, can_process=False)

    def test_cb_permissionsrequiredview(self):
        self.assertProcessorResults(views.PermissionRequiredView, can_process=False)

    def test_cb_userpassestestview(self):
        self.assertProcessorResults(views.UserPassesTestView, can_process=False)


class TestLoginRequiredDecoratorProcessor(DecoratorProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_decorators.LoginRequiredDecoratorProcessor()

    def test_base_view(self):
        self.assertProcessorResults(views.base_view, can_process=False)

    def test_login_required_view(self):
        self.assertProcessorResults(
            views.login_required_view,
            can_process=True,
            permissions=[],
            login_required=True,
            docstring=None
        )

    def test_permission_required_view(self):
        self.assertProcessorResults(views.permission_required_view, can_process=False)

    def test_staff_member_required_view(self):
        self.assertProcessorResults(views.staff_member_required_view, can_process=False)

    def test_active_user_required_view(self):
        self.assertProcessorResults(views.active_user_required_view, can_process=False)

    def test_anonymous_user_required_view(self):
        self.assertProcessorResults(views.anonymous_user_required_view, can_process=False)

    def test_superuser_required_view(self):
        self.assertProcessorResults(views.superuser_required_view, can_process=False)

    def test_user_passes_test_view(self):
        self.assertProcessorResults(views.user_passes_test_view, can_process=False)


class TestPermissionRequiredDecoratorProcessor(DecoratorProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_decorators.PermissionRequiredDecoratorProcessor()

    def test_base_view(self):
        self.assertProcessorResults(views.base_view, can_process=False)

    def test_login_required_view(self):
        self.assertProcessorResults(views.login_required_view, can_process=False)

    def test_permission_required_view(self):
        self.assertProcessorResults(
            views.permission_required_view,
            can_process=True,
            permissions=['tests.test_perm'],
            login_required=True,
            docstring=None
        )

    def test_permission_required_multi_view(self):
        """Multiple permissions passed to @permission_required should be retrieved."""
        self.assertProcessorResults(
            views.permission_required_multi_view,
            can_process=True,
            permissions=['tests.test_perm', 'tests.test_perm2'],
            login_required=True,
            docstring=None
        )

    def test_staff_member_required_view(self):
        self.assertProcessorResults(views.staff_member_required_view, can_process=False)

    def test_active_user_required_view(self):
        self.assertProcessorResults(views.active_user_required_view, can_process=False)

    def test_anonymous_user_required_view(self):
        self.assertProcessorResults(views.anonymous_user_required_view, can_process=False)

    def test_superuser_required_view(self):
        self.assertProcessorResults(views.superuser_required_view, can_process=False)

    def test_user_passes_test_view(self):
        self.assertProcessorResults(views.user_passes_test_view, can_process=False)


class StaffMemberRequiredDecoratorProcessor(DecoratorProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_decorators.StaffMemberRequiredDecoratorProcessor()

    def test_base_view(self):
        self.assertProcessorResults(views.base_view, can_process=False)

    def test_login_required_view(self):
        self.assertProcessorResults(views.login_required_view, can_process=False)

    def test_permission_required_view(self):
        self.assertProcessorResults(views.permission_required_view, can_process=False)

    def test_staff_member_required_view(self):
        self.assertProcessorResults(
            views.staff_member_required_view,
            can_process=True,
            permissions=[],
            login_required=True,
            docstring='Staff member required'
        )

    def test_active_user_required_view(self):
        self.assertProcessorResults(views.active_user_required_view, can_process=False)

    def test_anonymous_user_required_view(self):
        self.assertProcessorResults(views.anonymous_user_required_view, can_process=False)

    def test_superuser_required_view(self):
        self.assertProcessorResults(views.superuser_required_view, can_process=False)

    def test_user_passes_test_view(self):
        self.assertProcessorResults(views.user_passes_test_view, can_process=False)


class ActiveUserRequiredDecoratorProcessor(DecoratorProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_decorators.ActiveUserRequiredDecoratorProcessor()

    def test_base_view(self):
        self.assertProcessorResults(views.base_view, can_process=False)

    def test_login_required_view(self):
        self.assertProcessorResults(views.login_required_view, can_process=False)

    def test_permission_required_view(self):
        self.assertProcessorResults(
            views.permission_required_view, can_process=False)

    def test_staff_member_required_view(self):
        self.assertProcessorResults(
            views.staff_member_required_view,
            can_process=True,
            permissions=[],
            login_required=True,
            docstring='Active user required'
        )

    def test_active_user_required_view(self):
        self.assertProcessorResults(
            views.active_user_required_view,
            can_process=True,
            permissions=[],
            login_required=True,
            docstring='Active user required'
        )

    def test_anonymous_user_required_view(self):
        self.assertProcessorResults(views.anonymous_user_required_view, can_process=False)

    def test_superuser_required_view(self):
        self.assertProcessorResults(views.superuser_required_view, can_process=False)

    def test_user_passes_test_view(self):
        self.assertProcessorResults(views.user_passes_test_view, can_process=False)


class AnonymousUserRequiredDecoratorProcessor(DecoratorProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_decorators.AnonymousUserRequiredDecoratorProcessor()

    def test_base_view(self):
        self.assertProcessorResults(views.base_view, can_process=False)

    def test_login_required_view(self):
        self.assertProcessorResults(views.login_required_view, can_process=False)

    def test_permission_required_view(self):
        self.assertProcessorResults(
            views.permission_required_view, can_process=False)

    def test_staff_member_required_view(self):
        self.assertProcessorResults(views.staff_member_required_view, can_process=False)

    def test_active_user_required_view(self):
        self.assertProcessorResults(views.active_user_required_view, can_process=False)

    def test_anonymous_user_required_view(self):
        self.assertProcessorResults(
            views.anonymous_user_required_view,
            can_process=True,
            permissions=[],
            login_required=False,
            docstring='Anonymous user required'
        )

    def test_superuser_required_view(self):
        self.assertProcessorResults(views.superuser_required_view, can_process=False)

    def test_user_passes_test_view(self):
        self.assertProcessorResults(views.user_passes_test_view, can_process=False)


class SuperUserRequiredDecoratorProcessor(DecoratorProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_decorators.SuperUserRequiredDecoratorProcessor()

    def test_base_view(self):
        self.assertProcessorResults(views.base_view, can_process=False)

    def test_login_required_view(self):
        self.assertProcessorResults(views.login_required_view, can_process=False)

    def test_permission_required_view(self):
        self.assertProcessorResults(
            views.permission_required_view, can_process=False)

    def test_staff_member_required_view(self):
        self.assertProcessorResults(views.staff_member_required_view, can_process=False)

    def test_active_user_required_view(self):
        self.assertProcessorResults(views.active_user_required_view, can_process=False)

    def test_anonymous_user_required_view(self):
        self.assertProcessorResults(views.anonymous_user_required_view, can_process=False)

    def test_superuser_required_view(self):
        self.assertProcessorResults(
            views.superuser_required_view,
            can_process=True,
            permissions=[],
            login_required=True,
            docstring='Superuser required'
        )

    def test_user_passes_test_view(self):
        self.assertProcessorResults(views.user_passes_test_view, can_process=False)


class UserPassesTestDecoratorProcessor(DecoratorProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_decorators.UserPassesTestDecoratorProcessor()

    def test_base_view(self):
        self.assertProcessorResults(views.base_view, can_process=False)

    def test_login_required_view(self):
        self.assertProcessorResults(views.login_required_view, can_process=False)

    def test_permission_required_view(self):
        self.assertProcessorResults(
            views.permission_required_view, can_process=False)

    def test_staff_member_required_view(self):
        self.assertProcessorResults(views.staff_member_required_view, can_process=False)

    def test_active_user_required_view(self):
        self.assertProcessorResults(views.active_user_required_view, can_process=False)

    def test_anonymous_user_required_view(self):
        self.assertProcessorResults(views.anonymous_user_required_view, can_process=False)

    def test_superuser_required_view(self):
        self.assertProcessorResults(
            views.superuser_required_view, can_process=False)

    def test_user_passes_test_view(self):
        self.assertProcessorResults(
            views.user_passes_test_view,
            can_process=True,
            permissions=[],
            login_required=None,
            docstring='Custom user test'
        )



import inspect

from django.conf import ImproperlyConfigured


class BaseProcessor:

    def can_process(self, view):
        """
        Can this processor process the provided view?

        :param view: the view being processed.
        :type view: function or class
        :return: whether this processor can process the view.
                 Default: ``False``
        :rtype: boolean
        """
        return True

    def get_permission_required(self, view):
        """
        Get the permissions required on the provided view.
        Must return an iterable.

        :param view: the view being processed.
        :type view: function or class
        :return: the permissions required to access the view. Default: ``[]``
        :rtype: list(str)
        """
        return []

    def get_login_required(self, view):
        """
        Get whether or not the view needs the user to be logged in to access.

        :param view: the view being processed.
        :type view: function or class
        :return: whether a user must be logged in to access this view.
                 Default: ``False``
        :rtype: boolean or None (if unknown)
        """
        return False

    def get_docstring(self, view):
        """
        Return any additional information that should be displayed when
        showing permisison information.

        :param view: the view being processed.
        :type view: function or class
        :return: the string to display in the additional info column. Default: ``None``
        :rtype: str or None
        """
        return None


class BaseFuncViewProcessor(BaseProcessor):
    """Base class for processing function based views."""

    def can_process(self, view):
        return inspect.isfunction(view)


class BaseCBVProcessor(BaseProcessor):
    """Base class for processing class based views."""

    def can_process(self, view):
        return inspect.isclass(view)


class BaseFileredMixinProcessor(BaseCBVProcessor):
    """
    Base class for parsing mixins on class based views.
    Set ``class_filter`` to filter the class names the processor applies to.
    ONLY checks top level base classes.

    :var class_filter: initial value: ``None``
    """
    class_filter = None

    def can_process(self, view):
        if not super().can_process(view):
            return False

        view_bases = [cls.__module__ + '.' + cls.__name__ for cls in view.__bases__]

        for cls_filter in self.get_class_filter():
            if cls_filter in view_bases:
                return True

        return False

    def get_class_filter(self):
        """
        Override this method to override the class_names attribute.
        Must return an iterable.

        :return: a list of strings containing the full paths of mixins to detect.
        :raises ImproperlyConfigured: if the ``class_filter`` atribute is ``None``.
        """
        if self.class_filter is None:
            raise ImproperlyConfigured(
                '{0} is missing the class_filter attribute. Define {0}.class_filter, or override '
                '{0}.get_class_filter().'.format(self.__class__.__name__)
            )
        if isinstance(self.class_filter, str):
            cls_filter = (self.class_filter, )
        else:
            cls_filter = self.class_filter
        return cls_filter



from django.test import SimpleTestCase


class ProcessorTestCase(SimpleTestCase):
    processor = None

    def assertProcessorResults(self, view,
                               can_process=True, permissions=[],
                               login_required=False, docstring=None):

        can_process_result = self.processor.can_process(view)
        self.assertEqual(can_process_result, can_process)

        if can_process_result:
            self.assertCountEqual(self.processor.get_permission_required(view), permissions)
            self.assertEqual(self.processor.get_login_required(view), login_required)
            self.assertEqual(self.processor.get_docstring(view), docstring)



from django import forms
from django.contrib.admin import widgets
from django.contrib.auth import get_user_model
from django.contrib.auth.models import ContentType, Group, Permission

User = get_user_model()


class AuditorAdminPermissionForm(forms.ModelForm):
    name = forms.CharField(disabled=True)
    content_type = forms.ModelChoiceField(ContentType.objects.all(), disabled=True)
    codename = forms.CharField(disabled=True)
    users = forms.ModelMultipleChoiceField(
        widget=widgets.FilteredSelectMultiple("User", is_stacked=False),
        queryset=User.objects.filter(is_active=True),
        required=False,
    )
    groups = forms.ModelMultipleChoiceField(
        widget=widgets.FilteredSelectMultiple("Group", is_stacked=False),
        queryset=Group.objects.all(),
        required=False,
    )

    class Meta:
        model = Permission
        fields = (
            'name', 'content_type', 'codename',
            'users', 'groups',
        )
